module( ..., package.seeall )

local ut = require( 'udunits2' )

local time_t = { year   = 2011,
		 month  = 9,
		 day    = 12,
		 hour   = 13,
		 minute = 53,
		 second = 08 }

local time_double = 337528388

setup = _G.setup

function test_trim( )

   local trimmed = ut.trim( '\nbxxxx ', ut.ASCII )

   assert_equal( 'bxxxx', trimmed )

end

function    test_encode( )

   local time = ut.encode_time( time_t.year,
				time_t.month,
				time_t.day,
				time_t.hour,
				time_t.minute,
				time_t.second );

   local date = ut.encode_date( time_t.year, time_t.month, time_t.day )
   local clock = ut.encode_clock( time_t.hour, time_t.minute, time_t.second )

   assert_equal( time_double, time )
   assert_equal( date+clock, time )


end

function    test_decode_time( )

   local t = {}

   t.year, t.month, t.day, t.hour, t.minute, t.second
      = ut.decode_time( time_double )

   for k,v in pairs( time_t ) do

      assert_equal( v, t[k], k );
   end

end

function    test_status( )

   ut.set_status( ut.BAD_ARG )
   assert_equal( ut.BAD_ARG, ut.get_status( ) )


end

function test_enum_status( )

   local enum = {
      [0] = 'SUCCESS',
      'BAD_ARG',
      'EXISTS',
      'NO_UNIT',
      'OS',
      'NOT_SAME_SYSTEM',
      'MEANINGLESS',
      'NO_SECOND',
      'VISIT_ERROR',
      'CANT_FORMAT',
      'SYNTAX',
      'UNKNOWN',
      'OPEN_ARG',
      'OPEN_ENV',
      'OPEN_DEFAULT',
      'PARSE'
   }

   for v,k in pairs( enum ) do
      assert_equal( v, ut[k], k )
   end


end

function test_enum_encoding( )

   local enum = {
      ASCII = 0,
      ISO_8859_1 = 1,
      LATIN1 = 1,
      UTF8 = 2
   }

   for k,v in pairs( enum ) do
      assert_equal( v, ut[k], k )
   end

end

function test_error_messages_lua( )

   local stack = {}

   local func = function ( message )
		   stack[#stack+1] = message
		end

   local handler = ut.set_error_message_handler( func )

   assert_equal( _G.error_message_handler, handler );

   ut.handle_error_message( "message 1")
   ut.handle_error_message( "message 2")

   assert_equal( "message 1", stack[1] );
   assert_equal( "message 2", stack[2] );

   -- reset original handler
   local ok, msg = pcall( ut.set_error_message_handler, handler )
   assert_true( ok, msg )

end


function test_error_messages_C( )

   local ut_test = require( 'udunits2_test' )

   ut_test.set_error_message_handler( )

   local last = ut.set_error_message_handler( 'stderr' )

   assert_true( ut.isa( last, 'error_message_handler' ) )

end
